//$Id: vimos.dpr,v 1.11 2005/04/19 11:10:33 cizzo Exp $
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1999 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT Data Flow System
//AUTHOR:    R.Palsa, C.Izzo, P.Sartoretti - ESO/DMD/DPG
//SUBSYSTEM: VIMOS Instrument Pipeline
//PURPOSE:   Definition of reduction rules
//
//$Name: vmcdb-2_6_0 $
//$Revision: 1.11 $
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

instrument: "VIMOS";
name:       "$Id: vimos.dpr,v 1.11 2005/04/19 11:10:33 cizzo Exp $";
version:    "$Revision: 1.11 $";


// Reduction rules triggered by single frames

vmMaskToCcd(REF_IMAGE){
  if (t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE,
              MIN_RET=0, MAX_RET=1) &&
      mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) && 
      mdark=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK,
              MIN_RET=0, MAX_RET=0) && 
      mskyflat=gen_query(REF_IMAGE, DO_CATG=IMG_MASTER_SKY_FLAT,
              MIN_RET=0, MAX_RET=1))
    execute vmMaskToCcd(REF_IMAGE, t_ccd, mbias, mdark, mskyflat);
}


vmSkyCCD(REF_IMAGE){
  if (t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE,
              MIN_RET=0, MAX_RET=1) &&
      mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) && 
      mdark=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK,
              MIN_RET=0, MAX_RET=1) && 
      mskyflat=gen_query(REF_IMAGE, DO_CATG=IMG_MASTER_SKY_FLAT,
              MIN_RET=1, MAX_RET=1) &&
      t_photometry=gen_query(REF_IMAGE, DO_CATG=PHOTOMETRIC_TABLE,
              MIN_RET=1, MAX_RET=1) &&
      t_astrometry=gen_query(REF_IMAGE, DO_CATG=ASTROMETRIC_TABLE,
              MIN_RET=1, MAX_RET=1))  
    execute vmSkyCCD(REF_IMAGE, t_ccd, mbias, mdark, mskyflat, 
                     t_photometry, t_astrometry);
}


vmImObsStare(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) &&
      mdark=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK,
              MIN_RET=0, MAX_RET=1) &&
      mskyflat=gen_query(REF_IMAGE, DO_CATG=IMG_MASTER_SKY_FLAT,
              MIN_RET=1, MAX_RET=1) &&
      t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE,
              MIN_RET=0, MAX_RET=1) &&
      t_photometry=gen_query(REF_IMAGE, DO_CATG=PHOTOMETRIC_TABLE,
              MIN_RET=0, MAX_RET=1))
    execute vmImObsStare(REF_IMAGE, mbias, mdark, mskyflat,
                         t_ccd, t_photometry);
}


vmImStandard(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) &&
      mdark=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK,
              MIN_RET=0, MAX_RET=1) &&
      mskyflat=gen_query(REF_IMAGE, DO_CATG=IMG_MASTER_SKY_FLAT,
              MIN_RET=1, MAX_RET=1) &&
      t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE,
              MIN_RET=0, MAX_RET=1) &&
      t_phstd=gen_query(REF_IMAGE, DO_CATG=PHOTOMETRIC_CATALOG,
              MIN_RET=1, MAX_RET=1) &&
      t_photometry=gen_query(REF_IMAGE, DO_CATG=PHOTOMETRIC_TABLE,
              MIN_RET=1, MAX_RET=1))
    execute vmImStandard(REF_IMAGE, mbias, mdark, mskyflat,
                         t_ccd, t_phstd, t_photometry);
}


vmImPreimaging(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) &&
      mskyflat=gen_query(REF_IMAGE, DO_CATG=IMG_MASTER_SKY_FLAT,
              MIN_RET=1, MAX_RET=1) &&
      t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE,
              MIN_RET=0, MAX_RET=1) &&
      t_photometry=gen_query(REF_IMAGE, DO_CATG=PHOTOMETRIC_TABLE,
              MIN_RET=0, MAX_RET=1))
    execute vmImPreimaging(REF_IMAGE, mbias, mskyflat, t_ccd, t_photometry);
}


vmSpCalDisp(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) &&
      mdark=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK,
              MIN_RET=0, MAX_RET=0) &&
      scrflt=gen_query(REF_IMAGE, DO_CATG=MOS_MASTER_SCREEN_FLAT,
              MIN_RET=0, MAX_RET=1) &&
      lincat=gen_query(REF_IMAGE, DO_CATG=LINE_CATALOG,
              MIN_RET=1, MAX_RET=1) &&
      t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE,
              MIN_RET=0, MAX_RET=1) &&
      t_ifu=gen_query(REF_IMAGE, DO_CATG=IFU_TABLE,
              MIN_RET=0, MAX_RET=0) &&
      t_grism=gen_query(REF_IMAGE, DO_CATG=GRISM_TABLE,
              MIN_RET=1, MAX_RET=1))
    execute vmSpCalDisp(REF_IMAGE, mbias, mdark, scrflt, lincat, 
                        t_ccd, t_ifu, t_grism);
}


vmMosObsStare(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) &&
      mdark=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK,
              MIN_RET=0, MAX_RET=1) && 
      atm=gen_query(REF_IMAGE, DO_CATG=EXTINCT_TABLE,
              MIN_RET=0, MAX_RET=1) &&
      sphot=gen_query(REF_IMAGE, DO_CATG=MOS_SPECPHOT_TABLE,
              MIN_RET=0, MAX_RET=1) &&
      scrflt=gen_query(REF_IMAGE, DO_CATG=MOS_MASTER_SCREEN_FLAT, 
              MIN_RET=0, MAX_RET=1) &&
      t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE, 
              MIN_RET=1, MAX_RET=1) &&
      t_grism=gen_query(REF_IMAGE, DO_CATG=GRISM_TABLE, 
              MIN_RET=1, MAX_RET=1))
    execute vmMosObsStare(REF_IMAGE, mbias, mdark, atm, sphot, scrflt,
                          t_ccd, t_grism);
}


// Reduction rules triggered when a template is finished


vmDet(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1))
    execute vmDet(REF_IMAGE, mbias);
}


vmBias(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE,DO_CATG=MASTER_BIAS,
              MIN_RET=0, MAX_RET=0) &&
      t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE,
              MIN_RET=0, MAX_RET=1))
    execute vmBias(REF_IMAGE, mbias, t_ccd);
}


vmDark(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE,DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) &&
      t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE,
              MIN_RET=0, MAX_RET=1))
    execute vmDark(REF_IMAGE, mbias, t_ccd);
}


vmImFlatScreen(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) && 
      mdark=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK, 
              MIN_RET=0, MAX_RET=1) &&
      t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE,
              MIN_RET=0, MAX_RET=1))
    execute vmImFlatScreen(REF_IMAGE, mbias, mdark, t_ccd);
}


vmImFlatSky(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) && 
      mdark=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK, 
              MIN_RET=0, MAX_RET=1) &&
      mscrflat=gen_query(REF_IMAGE, DO_CATG=IMG_MASTER_SCREEN_FLAT,
              MIN_RET=0, MAX_RET=1) && 
      t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE,
              MIN_RET=0, MAX_RET=1))
    execute vmImFlatSky(REF_IMAGE, mbias, mdark, mscrflat, t_ccd);
}


vmImObsJitter(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) &&
      mdark=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK,
              MIN_RET=0, MAX_RET=1) &&
      mskyflat=gen_query(REF_IMAGE, DO_CATG=IMG_MASTER_SKY_FLAT,
              MIN_RET=1, MAX_RET=1) &&
      t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE,
              MIN_RET=0, MAX_RET=1) &&
      t_photometry=gen_query(REF_IMAGE, DO_CATG=PHOTOMETRIC_TABLE,
              MIN_RET=1, MAX_RET=1))
    execute vmImObsJitter(REF_IMAGE, mbias, mdark, mskyflat,
                          t_ccd, t_photometry);
}


vmSpFlat(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) &&
      mdark=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK,
              MIN_RET=0, MAX_RET=1) && 
      t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE, 
              MIN_RET=0, MAX_RET=1) &&
      t_ifu=gen_query(REF_IMAGE, DO_CATG=IFU_TABLE, 
              MIN_RET=0, MAX_RET=0) &&
      t_grism=gen_query(REF_IMAGE, DO_CATG=GRISM_TABLE, 
              MIN_RET=1, MAX_RET=1))
    execute vmSpFlat(REF_IMAGE, mbias, mdark, t_ccd, t_ifu, t_grism);
}


vmMosObsJitter(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) &&
      mdark=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK,
              MIN_RET=0, MAX_RET=1) && 
      atm=gen_query(REF_IMAGE, DO_CATG=EXTINCT_TABLE,
              MIN_RET=0, MAX_RET=1) &&
      sphot=gen_query(REF_IMAGE, DO_CATG=MOS_SPECPHOT_TABLE,
              MIN_RET=0, MAX_RET=1) &&
      scrflt=gen_query(REF_IMAGE, DO_CATG=MOS_MASTER_SCREEN_FLAT, 
              MIN_RET=0, MAX_RET=1) &&
      t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE, 
              MIN_RET=1, MAX_RET=1) &&
      t_grism=gen_query(REF_IMAGE, DO_CATG=GRISM_TABLE, 
              MIN_RET=1, MAX_RET=1))
    execute vmMosObsJitter(REF_IMAGE, mbias, mdark, atm, sphot, scrflt,
                           t_ccd, t_grism);
}


vmIfuCalib(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) &&
      lincat=gen_query(REF_IMAGE, DO_CATG=LINE_CATALOG,
              MIN_RET=1, MAX_RET=1) &&
      ident=gen_query(REF_IMAGE, DO_CATG=IFU_IDENT,
              MIN_RET=0, MAX_RET=1))
    execute vmIfuCalib(REF_IMAGE, mbias, lincat, ident);
}


vmIfuScience(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) &&
      atm=gen_query(REF_IMAGE, DO_CATG=EXTINCT_TABLE,
              MIN_RET=0, MAX_RET=1) &&
      sphot=gen_query(REF_IMAGE, DO_CATG=IFU_SPECPHOT_TABLE,
              MIN_RET=0, MAX_RET=1) &&
      ids=gen_query(REF_IMAGE, DO_CATG=IFU_IDS,
              MIN_RET=1, MAX_RET=1) &&
      trace=gen_query(REF_IMAGE, DO_CATG=IFU_TRACE,
              MIN_RET=1, MAX_RET=1) &&
      trans=gen_query(REF_IMAGE, DO_CATG=IFU_TRANSMISSION,
              MIN_RET=1, MAX_RET=1))
    execute vmIfuScience(REF_IMAGE, mbias, atm, sphot, ids, trace, trans);
}


vmIfuStandard(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) &&
      std=gen_query(REF_IMAGE, DO_CATG=STD_FLUX_TABLE,
              MIN_RET=1, MAX_RET=1) &&
      atm=gen_query(REF_IMAGE, DO_CATG=EXTINCT_TABLE,
              MIN_RET=1, MAX_RET=1) &&
      ids=gen_query(REF_IMAGE, DO_CATG=IFU_IDS,
              MIN_RET=1, MAX_RET=1) &&
      trace=gen_query(REF_IMAGE, DO_CATG=IFU_TRACE,
              MIN_RET=1, MAX_RET=1) &&
      trans=gen_query(REF_IMAGE, DO_CATG=IFU_TRANSMISSION,
              MIN_RET=1, MAX_RET=1))
    execute vmIfuStandard(REF_IMAGE, mbias, std, atm, ids, trace, trans);
}


vmMosStandard(REF_IMAGE){
  if (mbias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS,
              MIN_RET=1, MAX_RET=1) &&
      mdark=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK,
              MIN_RET=0, MAX_RET=1) &&
      std=gen_query(REF_IMAGE, DO_CATG=STD_FLUX_TABLE,
              MIN_RET=1, MAX_RET=1) &&
      atm=gen_query(REF_IMAGE, DO_CATG=EXTINCT_TABLE,
              MIN_RET=1, MAX_RET=1) &&
      scrflt=gen_query(REF_IMAGE, DO_CATG=MOS_MASTER_SCREEN_FLAT,
              MIN_RET=0, MAX_RET=1) &&
      t_ccd=gen_query(REF_IMAGE, DO_CATG=CCD_TABLE, 
              MIN_RET=1, MAX_RET=1) &&
      t_grism=gen_query(REF_IMAGE, DO_CATG=GRISM_TABLE,
              MIN_RET=1, MAX_RET=1))
    execute vmMosStandard(REF_IMAGE, mbias, mdark, std, atm, scrflt,
                          t_ccd, t_grism);
}
